/**********************************************************************************
Note:  I've set the tabs to 4 spaces in my editor.

Progam Purpose:
  This little program tests a rumour I heard that Watcom (or Dos4GW?) doesn't
  consolidate free memory in 32bit DOS mode using Dos4GW.

Example:
	Here's an example to explain what I mean when I say "consolidate free memory":
  -Let's say we have 100 bytes of memory total in our machine.  (maybe it's an 8088?)
  -The largest block of continuous memory we can allocate is 100 bytes.
  -If we allocate 10 blocks each of which is 10bytes, we will have zero bytes left.
  -Now let's free all 10 of our imaginary blocks of memory, leaving us with 100 bytes
   free.  Ideally we should now be able to alloacte all 100 bytes as a single block.
   That's becuase the memory was "consolidated" when it was freed.
  -Unfortuatly Watcom doesn't seem to do this!  In our example, although 100 bytes of
   memory is availible, the largest block of memory we can alocate is 10 bytes!!!
   This is a real problem, and effectivly means that we should all develop our own
   memory management libraries.  Gross.

Plea:
	I really hope that I'm wrong!  If any of you experts out there can see a major
	mistake in my test program that would prove Watcom (Dos4GW?) does consolidate
	free memory I'd love to hear from you,

	Note that I learned how to get the largest availible block from the Watcom
	on-line help, Programmers Guide.  If I'm doing something wrong it might be
	because their documentation is incorrect?

	I've sent this to Watcom, and I'll upload any responce I get from them.

Program Usage:
	I've added a small portion of code from my General.hpp library that allows for
	easy decoding of command line arguments.  Here's the arguments this program
	uses:
	-n:###		The -n option sets the number of blocks that are allocated.
				Defaults to 100.
	-s:###		the -s option sets the size of each block that is allocated.
				Defaults to 65535.

Me:
	Feel free to pass this code on to all your friends, to use anyway you like.  If
	you do find out what's going on, or think that I'm right and Watcom (or Dos4GW?)
	doesn't consolidate free memory please e-mail me with your findings.
	I assume I should mention that I take no responsiblity for maintaining this code,
	or any damage that it causes to your system.  (yeah, like that'll happen...)
	
   Robert Alexander McCarter
   ramccart@undergrad.math.uwaterloo.ca
***********************************************************************************/

#include      <dos.h>
#include    <ctype.h>		// for toupper/tolower
#include   <stdlib.h>
#include <iostream.h>


/** Defines ************************************************************/
#define DPMI_INT        0x31
#define MIN(a,b)		(((a) < (b)) ? (a) : (b))

/** Fns Definitions ****************************************************/
unsigned GetLargestFreeMemoryBlock();				// Largest Free Block of Memory
float NumberCommandLine(int argc,char* argv[],char SearchFor,float Default=-1,int CaseMatters=0);


/** Fns ****************************************************************/
main(int argc,char *argv[])
{
	cout << "MemTest: Does Watcom/DOS4GW Consolidate Free Memory?"		<< endl;
	cout << "Usage:  -n  Number of test blocks to allocate."			<< endl;
	cout << "        -s  Size of each test block."						<< endl;

	char* TestBlocks[1024];
	unsigned SizeOfBlocks = NumberCommandLine(argc,argv,'s',65535);
	unsigned NumberBlocks = MIN( 1024, NumberCommandLine(argc,argv,'n',100) );

    unsigned InitLargestFree	= GetLargestFreeMemoryBlock();
	cout << endl << "Starting Test____________________________"			<< endl;
	cout << "Number Blocks      : " << NumberBlocks						<< endl;
	cout << "Size Of Blocks     : " << SizeOfBlocks						<< endl;
	cout << "Largest Free Block : " << InitLargestFree					<< endl;

		/*********** Allocate, then Deallocate some memory ************/
	for(int co=0;co<NumberBlocks;co++)				// Get some memory
		TestBlocks[co] = new char[ SizeOfBlocks ];
		
	for(co=0;co<NumberBlocks;co++)					// Free all the memory
		if(TestBlocks[co]!=NULL) delete[](TestBlocks[co]);

		/********* Get Statistics and Calculations about Memory ********/
	unsigned TotalMemAllocated	 = NumberBlocks*SizeOfBlocks;
	unsigned EndLargestFree		 = GetLargestFreeMemoryBlock();
	unsigned DifferenceBlockSize = InitLargestFree - EndLargestFree;
	unsigned Calculation1		 = DifferenceBlockSize - TotalMemAllocated;
	unsigned Calculation2		 = Calculation1 % 1024;

	cout << endl << "Finished Test____________________________"			<< endl;
	cout << "Total Memory Allocated for Test = " << TotalMemAllocated	<< endl;
	cout << "Largest Free Block After Test   = " << EndLargestFree		<< endl;
	cout << "Difference in Largest Block     = " << DifferenceBlockSize	<< endl;
	if( DifferenceBlockSize )
	{	cout << "DifBlockSize-TotalMemoryForTest = " << Calculation1	<< endl;
		cout << "Ditto Mod 1024                  = " << Calculation2	<< endl;
	}//if
}//main


struct RMemInfo								// See the Programmers Guide For more
{	unsigned LargestBlockAvail;				// info on this structure.  I copied it
	unsigned MaxUnlockedPage;				// directly from there, so no chances
	unsigned LargestLockablePage;			// of a mistake!
	unsigned LinAddrSpace;
	unsigned NumFreePagesAvail;
	unsigned NumPhysicalPagesFree;
	unsigned TotalPhysicalPages;
	unsigned FreeLinAddrSpace;
	unsigned SizeOfPageFile;
	unsigned Reserved[3];
} MemInfo;	//RMemInfo

		// Return the Largest Free Block of Memory
unsigned GetLargestFreeMemoryBlock()
{
	union REGS regs;
	struct SREGS sregs;
	regs.x.eax = 0x00000500;
	memset( &sregs, 0, sizeof(sregs) );

	sregs.es	= FP_SEG( &MemInfo );
	regs.x.edi	= FP_OFF( &MemInfo );
	int386x( DPMI_INT, &regs, &regs, &sregs );
	
	return( MemInfo.LargestBlockAvail );
}//LargestFreeMemoryBlock



	// Search Command line for parameter, return the position
	// (ex, 0th,5th) that it occurs on the command line, or 0 if !found.
int   IndexOnCommandLine(int argc,char* argv[],char SearchFor,int CaseMatters)
{
    char  Dash,Option;
    char  upper = (char)toupper(SearchFor),	// upper case
    	  lower = (char)tolower(SearchFor);	// lower case
    
    for(int co=1;co<argc;co+=1)			// Scan all arguments
    {	Dash   = argv[co][0];
		Option = argv[co][1];
		if( (Dash=='-' || Dash=='/') )		// an option
		    if( ( CaseMatters && Option==SearchFor) ||
		    	(!CaseMatters && (Option==upper || Option==lower) ))
		    	return( co );			// found - tell user #
    }//for
    return(0);
}//IndexOnCommandLine


	// Search Command line for parameter, return * to string (skipped over
	// the dash and parameter) if found, NULL if not found
char* StringCommandLine(int argc,char* argv[],char SearchFor,char *Default,int CaseMatters)
{
    int co=IndexOnCommandLine(argc,argv,SearchFor,CaseMatters);
    int AddressPos=2;							// skip first 2 chars
    if(co>0)									// found on command line
    {	if( argv[co][AddressPos]==':' )			// skip the optional ':'
	    AddressPos++;							// so skip 3 chars
		return( &(argv[co][AddressPos]));		// return the string
    }//if
    return( Default );
}


	// Search Command line for parameter, return the # associated with the
	// parameter.  -1 if parameter not found or error converting #
float NumberCommandLine(int argc,char* argv[],char SearchFor,float Default,int CaseMatters)
{
    char *StringInt = StringCommandLine(argc,argv,SearchFor,NULL,CaseMatters);
    if( StringInt==NULL ) return(Default);		// not found
    float RetValue = atof(StringInt);			// convert the string
    if( errno!=0 ) return( Default );			// an error has occured
    return( RetValue );							// return the value
}


/*EOF*/   
